from .return_class import AbstractApiClass


class DomainPurchaseResult(AbstractApiClass):
    """
        Domain purchase result

        Args:
            client (ApiClient): An authenticated API Client instance
            domain (str): The domain name being purchased.
            billedCredits (int): The total credits used for the purchase.
    """

    def __init__(self, client, domain=None, billedCredits=None):
        super().__init__(client, None)
        self.domain = domain
        self.billed_credits = billedCredits
        self.deprecated_keys = {}

    def __repr__(self):
        repr_dict = {f'domain': repr(
            self.domain), f'billed_credits': repr(self.billed_credits)}
        class_name = "DomainPurchaseResult"
        repr_str = ',\n  '.join([f'{key}={value}' for key, value in repr_dict.items(
        ) if getattr(self, key, None) is not None and key not in self.deprecated_keys])
        return f"{class_name}({repr_str})"

    def to_dict(self):
        """
        Get a dict representation of the parameters in this class

        Returns:
            dict: The dict value representation of the class parameters
        """
        resp = {'domain': self.domain, 'billed_credits': self.billed_credits}
        return {key: value for key, value in resp.items() if value is not None and key not in self.deprecated_keys}
